@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.131 2005/08/04 19:56:41 nickm Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.131 2005/08/04 19:56:41 nickm Exp $";
+/* $Id: circuitbuild.c,v 1.132 2005/08/07 21:24:00 nickm Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.132 2005/08/07 21:24:00 nickm Exp $";
 
 /**
  * \file circuitbuild.c
@@ -1427,8 +1427,8 @@ choose_good_entry_server(cpath_build_state_t *state)
 
     for (i=0; i < smartlist_len(rl->routers); i++) {
       r = smartlist_get(rl->routers, i);
-      if (!smartlist_string_num_isin(options->FirewallPorts, r->or_port))
-         smartlist_add(excluded, r);
+      if (!fascist_firewall_allows_address(options,r->addr,r->or_port))
+        smartlist_add(excluded, r);
     }
   }
   choice = router_choose_random_node(options->EntryNodes, options->ExcludeNodes,
