@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.391 2005/08/04 18:29:01 nickm Exp $ */
-const char connection_c_id[] = "$Id: connection.c,v 1.391 2005/08/04 18:29:01 nickm Exp $";
+/* $Id: connection.c,v 1.393 2005/08/07 19:27:38 nickm Exp $ */
+const char connection_c_id[] = "$Id: connection.c,v 1.393 2005/08/07 19:27:38 nickm Exp $";
 
 /**
  * \file connection.c
@@ -253,6 +253,10 @@ connection_free(connection_t *conn)
     if (conn->state == OR_CONN_STATE_OPEN)
       directory_set_dirty();
   }
+  if (conn->type == CONN_TYPE_CONTROL) {
+    conn->event_mask = 0;
+    control_update_global_event_mask();
+  }
   connection_unregister(conn);
   _connection_free(conn);
 }
@@ -272,6 +276,13 @@ connection_free_all(void)
   connection_t **carray;
 
   get_connection_array(&carray,&n);
+
+  /* We don't want to log any messages to controllers. */
+  for (i=0;i<n;i++)
+    if (carray[i]->type == CONN_TYPE_CONTROL)
+      carray[i]->event_mask = 0;
+  control_update_global_event_mask();
+
   for (i=0;i<n;i++)
     _connection_free(carray[i]);
 
