@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.112 2005/08/04 19:56:41 nickm Exp $ */
-const char control_c_id[] = "$Id: control.c,v 1.112 2005/08/04 19:56:41 nickm Exp $";
+/* $Id: control.c,v 1.114 2005/08/07 19:27:38 nickm Exp $ */
+const char control_c_id[] = "$Id: control.c,v 1.114 2005/08/07 19:27:38 nickm Exp $";
 
 /**
  * \file control.c
@@ -127,7 +127,6 @@ static char authentication_cookie[AUTHENTICATION_COOKIE_LEN];
 
 static void connection_printf_to_buf(connection_t *conn, const char *format, ...)
   CHECK_PRINTF(2,3);
-static void update_global_event_mask(void);
 static void send_control0_message(connection_t *conn, uint16_t type,
                                  uint32_t len, const char *body);
 static void send_control_done(connection_t *conn);
@@ -204,8 +203,8 @@ log_severity_to_event(int severity)
 
 /** Set <b>global_event_mask</b> to the bitwise OR of each live control
  * connection's event_mask field. */
-static void
-update_global_event_mask(void)
+void
+control_update_global_event_mask(void)
 {
   connection_t **conns;
   int n_conns, i;
@@ -524,6 +523,7 @@ send_control0_event(uint16_t event, uint32_t len, const char *body)
   get_connection_array(&conns, &n_conns);
   for (i = 0; i < n_conns; ++i) {
     if (conns[i]->type == CONN_TYPE_CONTROL &&
+        !conns[i]->marked_for_close &&
         conns[i]->state == CONTROL_CONN_STATE_OPEN_V0 &&
         conns[i]->event_mask & (1<<event)) {
       send_control0_message(conns[i], CONTROL0_CMD_EVENT, buflen, buf);
@@ -561,6 +561,7 @@ send_control1_event(uint16_t event, const char *format, ...)
   get_connection_array(&conns, &n_conns);
   for (i = 0; i < n_conns; ++i) {
     if (conns[i]->type == CONN_TYPE_CONTROL &&
+        !conns[i]->marked_for_close &&
         conns[i]->state == CONTROL_CONN_STATE_OPEN_V1 &&
         conns[i]->event_mask & (1<<event)) {
       connection_write_to_buf(buf, len, conns[i]);
@@ -847,7 +848,7 @@ handle_control_setevents(connection_t *conn, uint32_t len, const char *body)
   }
   conn->event_mask = event_mask;
 
-  update_global_event_mask();
+  control_update_global_event_mask();
   send_control_done(conn);
   return 0;
 }
