@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.239 2005/07/22 17:32:25 nickm Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.239 2005/07/22 17:32:25 nickm Exp $";
+/* $Id: directory.c,v 1.241 2005/08/08 17:32:17 nickm Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.241 2005/08/08 17:32:17 nickm Exp $";
 
 #include "or.h"
 
@@ -137,9 +137,8 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
       /* Pay attention to fascistfirewall when we're uploading a
        * router descriptor, but not when uploading a service
        * descriptor -- those use Tor. */
-      if (get_options()->FascistFirewall && purpose == DIR_PURPOSE_UPLOAD_DIR &&
-          !get_options()->HttpProxy) {
-        if (!smartlist_string_num_isin(get_options()->FirewallPorts, ds->dir_port))
+      if (purpose == DIR_PURPOSE_UPLOAD_DIR && !get_options()->HttpProxy) {
+        if (!fascist_firewall_allows_address(get_options(),ds->addr,ds->dir_port))
           continue;
       }
       directory_initiate_command_trusted_dir(ds, purpose, purpose_is_private(purpose),
@@ -838,7 +837,7 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_INFO,"Uploading rendezvous descriptor: finished with status 200 (\"%s\")", reason);
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver. Malformed rendezvous descriptor?", reason);
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s'. Malformed rendezvous descriptor?", reason, conn->address);
         break;
       default:
         log_fn(LOG_WARN,"http status %d (\"%s\") response unexpected (server '%s').", status_code, reason, conn->address);
