@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.634 2005/08/04 19:56:41 nickm Exp $ */
+/* $Id: or.h,v 1.636 2005/08/07 21:24:00 nickm Exp $ */
 
 /**
  * \file or.h
@@ -11,7 +11,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.634 2005/08/04 19:56:41 nickm Exp $"
+#define OR_H_ID "$Id: or.h,v 1.636 2005/08/07 21:24:00 nickm Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -1094,6 +1094,7 @@ typedef struct {
   int RunAsDaemon; /**< If true, run in the background. (Unix only) */
   int FascistFirewall; /**< Whether to prefer ORs reachable on open ports. */
   smartlist_t *FirewallPorts; /**< Which ports our firewall allows (strings). */
+  smartlist_t *FirewallIPs; /**< Which IPs our firewall allows (strings). */
   /** Application ports that require all nodes in circ to have sufficient uptime. */
   smartlist_t *LongLivedPorts;
   /** Should we try to reuse the same exit node for a given host */
@@ -1375,6 +1376,9 @@ int or_state_save(void);
 
 int config_getinfo_helper(const char *question, char **answer);
 
+int fascist_firewall_allows_address(or_options_t *options, uint32_t addr,
+                                    uint16_t port);
+
 /********************************* connection.c ***************************/
 
 const char *conn_type_to_string(int type);
@@ -1536,6 +1540,7 @@ typedef enum or_conn_status_event_t {
   OR_CONN_EVENT_CLOSED       = 3,
 } or_conn_status_event_t;
 
+void control_update_global_event_mask(void);
 void control_adjust_event_log_severity(void);
 void disable_control_logging(void);
 void enable_control_logging(void);
@@ -1652,6 +1657,7 @@ void hibernate_begin_shutdown(void);
 int we_are_hibernating(void);
 void consider_hibernation(time_t now);
 int accounting_getinfo_helper(const char *question, char **answer);
+void accounting_set_bandwidth_usage_from_state(or_state_t *state);
 
 /********************************* main.c ***************************/
 
