@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerlist.c,v 1.245 2005/07/22 21:12:10 nickm Exp $ */
-const char routerlist_c_id[] = "$Id: routerlist.c,v 1.245 2005/07/22 21:12:10 nickm Exp $";
+/* $Id: routerlist.c,v 1.246 2005/08/07 21:24:00 nickm Exp $ */
+const char routerlist_c_id[] = "$Id: routerlist.c,v 1.246 2005/08/07 21:24:00 nickm Exp $";
 
 /**
  * \file routerlist.c
@@ -187,7 +187,8 @@ router_pick_directory_server_impl(int requireother, int fascistfirewall,
     if (requireother && router_is_me(router))
       continue;
     if (fascistfirewall) {
-      if (!smartlist_string_num_isin(get_options()->FirewallPorts, router->dir_port))
+      if (!fascist_firewall_allows_address(get_options(),router->addr,
+                                           router->dir_port))
         continue;
     }
     /* before 0.0.9rc5-cvs, only trusted dirservers served status info. */
@@ -230,7 +231,7 @@ router_pick_trusteddirserver_impl(int requireother, int fascistfirewall)
           !memcmp(me->identity_digest, d->digest, DIGEST_LEN))
         continue;
       if (fascistfirewall) {
-        if (!smartlist_string_num_isin(get_options()->FirewallPorts, d->dir_port))
+        if (!fascist_firewall_allows_address(get_options(),d->addr,d->dir_port))
           continue;
       }
       smartlist_add(sl, d);
